// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef MARBLE_VERSION_H
#define MARBLE_VERSION_H

#define MARBLE_VERSION_STRING "26.04.2"
#define MARBLE_VERSION_MAJOR 26
#define MARBLE_VERSION_MINOR 4
#define MARBLE_VERSION_PATCH 2
#define MARBLE_VERSION ((26<<16)|(4<<8)|(2))

#endif
