<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="marble_part" version="17">

<MenuBar>
  <Menu name="file" noMerge="1">
    <text>&amp;File</text>
    <Action name="file_open"/>
    <Action name="file_open_recent"/>
    <Action name="new_stuff"/>
    <Action name="createMap"/>
    <Action name="file_download_region"/>
    <Action name="exportMap"/>
    <Separator/>
    <Action name="file_print"/>
    <Action name="file_print_preview"/>
    <Separator/>
    <Action name="workOffline"/>
    <Action name="file_quit"/>
  </Menu>
  <Menu name="edit" noMerge="1">
    <text>&amp;Edit</text>
    <Action name="edit_copy"/>
    <Action name="edit_copy_coordinates"/>
    <Separator/>
    <Action name="external_editor"/>
    <Separator/>
    <Action name="record_movie" />
    <Action name="stop_recording" />
  </Menu>
  <Menu name="view" noMerge="1">
    <text>&amp;View</text>
    <Action name="show_crosshairs" group="show_merge"/>
    <Action name="view_redisplay"/>
    <Separator/>
    <Menu name="infoboxes" noMerge="1">
      <text>&amp;Info Boxes</text>
      <Action name="options_lock_floatitems"/>
      <Separator/>
      <ActionList name="infobox_actionlist" />
    </Menu>
    <Menu name="onlineservices" noMerge="1">
      <text>&amp;Online Services</text>
      <ActionList name="onlineservices_actionlist" />
    </Menu>
    <Separator/>
    <Action name="show_clouds"/>
    <ActionList name="themerender_actionlist"/>
    <Separator/>
    <Action name="control_sun"/>
    <Action name="control_time"/>
    <Separator/>
    <ActionList name="plugins_menuactionlist" />
  </Menu>
  <Menu name="settings" noMerge="1">
    <text>&amp;Settings</text>
    <Merge name="StandardToolBarMenuHandler" />
    <Separator/>
    <Menu name="panels" noMerge="1">
      <text>&amp;Panels</text>
      <ActionList name="panels_actionlist" />
    </Menu>
    <Action name="options_show_statusbar"/>
    <Separator/>
    <Menu name="viewSize" noMerge="1">
      <text>&amp;View Size</text>
      <ActionList name="viewSize_actionlist" />
    </Menu>
    <Action name="fullscreen"/>
    <Separator/>
    <Action name="options_configure" append="configure_merge"/>
  </Menu>
  <Menu name="bookmarks" noMerge="1">
      <text>&amp;Bookmarks</text>
      <Action name="add_bookmark"/>
      <Action name="show_bookmarks"/>
      <Action name="set_home"/>
      <Action name="manage_bookmarks"/>
      <Separator/>
      <ActionList name="folders"/>
  </Menu>
</MenuBar>

<ToolBar position="Top" noMerge="1" hidden="true" name="mainToolBar">
  <text>Main Toolbar</text>
  <Action name="file_open"/>
  <Action name="file_print"/>
  <Separator/>
  <Action name="edit_copy"/>
  <Separator/>
<!--
  <Action name="sun_shadow"/>
  <Action name="sun_zenith"/>
-->
</ToolBar>
<!--
<ToolBar position="Top" hidden="true" name="pluginToolBar">
  <text>Edit Toolbar</text>
  <ActionList name="plugins_actionlist" />
</ToolBar>
-->
</kpartgui>
